/*
 * power_manage.c
 *
 *  Created on: Aug 4, 2016
 *      Author: VENSIINDIA
 */

#include "sensor_conditioner.h"
#include "board_config.h"
#include "custom_app_uart.h"
#include "app_timer.h"

#define UART_TX_BUF_SIZE                 	256
#define UART_RX_BUF_SIZE                	16

#define SENSOR_CONDITIONER_TIME_INTERVAL    APP_TIMER_TICKS(30, 0)

static uint8_t parity_type = SENSOR_PARITY_EVEN;
static sensor_cndtnr_event_handler_t m_sensor_cndtnr_handler;

APP_TIMER_DEF( m_sh35_uart_timer_id);

void sensor_condtnr_start(void) {
	PRESSURE_SET;
}

void sensor_condtnr_stop(void) {
	PRESSURE_RESET;
}

static uint32_t extract_data_byte(uint8_t *pressure_byte, uint8_t *buffer) {
	uint8_t data = 0;
	uint8_t parity = 0;

	for (int i = 0; i < 8; i++) {
		if (buffer[i] > 0xF0) {
			parity++;
			data <<= 1;
			data |= 1;
		} else {
			data <<= 1;
		}
	}
	//checking parity bit with offset 8
	if (buffer[8] > 0xF0) {
		parity++;
	}

	if (parity_type == SENSOR_PARITY_EVEN) {
	 if (parity & 0x01)
	 return SENSOR_ERROR_PARITY;
	 } else {
	 if (!(parity & 0x01))
	 return SENSOR_ERROR_PARITY;
	 }

	*pressure_byte = data;
	return SENSOR_SUCCESS;
}

static void uart_event_handle(app_uart_evt_t * p_event) {

	switch (p_event->evt_type) {
	case APP_UART_DATA_READY:
		app_timer_start(m_sh35_uart_timer_id,
		SENSOR_CONDITIONER_TIME_INTERVAL, NULL);
		break;

	case APP_UART_FIFO_ERROR:
		app_uart_flush();
		break;

	case  APP_UART_COMMUNICATION_ERROR:
		app_uart_flush();
		break;
	default:
		break;
	}
}

static void sensor_uart_init(void) {
	ret_code_t err_code;

	const app_uart_comm_params_t comm_params = {
	RX_PIN_NUMBER,
	TX_PIN_NUMBER, 0, 0, APP_UART_FLOW_CONTROL_DISABLED, false,
	UART_BAUDRATE_BAUDRATE_Baud9600 };

	APP_UART_FIFO_INIT(&comm_params,
	UART_RX_BUF_SIZE,
	UART_TX_BUF_SIZE, uart_event_handle, APP_IRQ_PRIORITY_LOW, err_code);

	APP_ERROR_CHECK(err_code);
}

static void uart_meas_timeout_handler(void * p_context) {
	uint8_t uart_data;
	uint8_t data_buffer[20], len;
	uint32_t err_code;
	uint8_t p_value[2];
	int16_t pressure_value = 0;

	memset(data_buffer, 0, 20);
	for (len = 0; len < 20; len++) {
		err_code = app_uart_get(&uart_data);
		if (err_code == NRF_ERROR_NOT_FOUND) {
			break;
		}
		data_buffer[len] = uart_data;
	}


	if (len == 20) {
		err_code = extract_data_byte(&p_value[0],
				&data_buffer[SENSOR_FIRST_BYTE_OFFSET]);
		if (err_code == SENSOR_SUCCESS) {
			err_code = extract_data_byte(&p_value[1],
				&data_buffer[SENSOR_SECOND_BYTE_OFFSET]);
			if (err_code == SENSOR_SUCCESS) {

				//converting two bytes of read data into 16 byte form
				pressure_value = p_value[0];
				pressure_value <<= 8;
				pressure_value |= p_value[1];

				sensor_cndtnr_event_t event;
				event.event_type = SENSOR_CONDITIONER_EVT_VALUE;
				event.sensor_cndtnr_value = pressure_value;

				m_sensor_cndtnr_handler(event);
			}
		}
	}

	app_uart_flush();
}

/**
 * @brief ADC initialization.
 */
void sensor_condtnr_init(sensor_cndtnr_event_handler_t evt_handler) {
	PRESSURE_INIT_PIN(PRESSURE_SENSOR_RESET_PIN);
	PRESSURE_SET;

	m_sensor_cndtnr_handler = evt_handler;
	sensor_uart_init();

	uint32_t err_code = app_timer_create(&m_sh35_uart_timer_id,
			APP_TIMER_MODE_SINGLE_SHOT, uart_meas_timeout_handler);
	APP_ERROR_CHECK(err_code);
}
